package com.mdp.voice;

import java.net.DatagramPacket;
import java.net.DatagramSocket;
import java.net.InetAddress;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.TargetDataLine;

public class AudioReceiver extends Thread {

    private DatagramSocket dgSocket = null;
    private SourceDataLine sourceDataLine;
    private TargetDataLine targetDataLine;
    private AudioFormat.Encoding encoding = AudioFormat.Encoding.PCM_SIGNED;
    private AudioFormat audioFormat;
    private int iBufferSize = 10240 * 1;
    private float rate = 11025.0f;
    private int sampleSize = 16;
    private int channels = 1;
    private boolean bigEndian = true;
    private int port = 0;
    private static String strIP = null;
    private Globals globals;
    private static Mixer gMixer = null;
    private Mixer.Info[] mixerInfo = null;

    /**
     * AudioReceiver 
     * 
     * Read data from the network and play it through the speaker.
     * 
     * @param globals
     */
    public AudioReceiver(Globals globals) {
        this.port = Globals.getVOICE_PORT();
        this.globals = globals;
    }

    /**
     * Run method... read from the network and play through the speaker
     */
    public void run() {
        long lBytesRead = 0;
        long lStart = System.currentTimeMillis();

        try {

            mixerInfo = AudioSystem.getMixerInfo();

//			mixer = AudioSystem.getMixer(mixerInfo[ 1 ]);

            dgSocket = new DatagramSocket(port);

            /*
             * Audiosender reads from the Microphone and send the data out. 
             */
            AudioSender as = new AudioSender();
            as.setIP(strIP);
            as.setPort(port);
            as.start();


            /*
             * sourceXXX is the 'output'
             * targetXXX is the 'input'
             */
            DataLine.Info sourceLineInfo = new DataLine.Info(
                    SourceDataLine.class, getFormat(), iBufferSize);

            DataLine.Info targetLineInfo = new DataLine.Info(
                    TargetDataLine.class, getFormat(), iBufferSize);


            sourceDataLine = (SourceDataLine) AudioSystem.getLine(sourceLineInfo);


            // get target (i.e. MIC.. from a supported mixer
//            targetDataLine = getMixerTarget(targetLineInfo);

            targetDataLine = (TargetDataLine) AudioSystem.getLine(targetLineInfo);

            sourceDataLine.open();
            targetDataLine.open();

            sourceDataLine.start();
            targetDataLine.start();

            byte buffer[] = new byte[sourceDataLine.getBufferSize()];

            InetAddress address = InetAddress.getByName(strIP);

            byte[] byteData = new byte[iBufferSize];
            int buffersize = byteData.length;

            DatagramPacket packet = new DatagramPacket(byteData, buffersize,
                    address, port);

            dgSocket.setSoTimeout(2500);

            while (!globals.isHangup()) {
                /*
                 * Read from microphone
                 */
                int read = targetDataLine.read(buffer, 0, buffer.length);

                packet = new DatagramPacket(buffer, read, address, port);

                try {
                    dgSocket.receive( packet );
                    //System.out.print("Received packet: " + packet.getLength());
                    lBytesRead += packet.getLength();
                } catch (Exception e1) {
                    e1.printStackTrace(System.out);
                    dgSocket.close();
                    dgSocket = null;
                    dgSocket = new DatagramSocket(port);
                    globals.setHangup(true);
                    globals.getStatusScreen().dispose();
                    break;
                }

                byte[] b = packet.getData();

                //////////////////////////////////////////////
                //
                // Play it through the speaker
                // NOTE: we continue to read to ensure the
                // socket remains open.
                //
                ///////////////////////////////////////////////
                sourceDataLine.write(b, 0, b.length);

                long lElapsed = (System.currentTimeMillis() - lStart) / 1000;
                if (lElapsed > 0) {
                    globals.getStatusScreen().setStatusIn("Bytes in per second: "
                            + lBytesRead / lElapsed);
                    globals.getStatusScreen().setStatusElapsed("Elapsed: " + lElapsed);
                }
                Thread.yield();
            }
            
            dgSocket.close();
            
            System.exit(0);
            
        } catch (Exception e) {
            e.printStackTrace(System.out);

        } finally {
            try {
                dgSocket.close();
            } catch (Exception e) {
            }
        }
    }

    /**
     * Set ip address
     * @param s
     */
    public void setIP(String s) {
        strIP = s;
    }

    /**
     * Return an Audioformat object.
     * 
     * @return
     */
    private AudioFormat getFormat() {
        return new AudioFormat(encoding, rate, sampleSize, channels,
                (sampleSize / 8) * channels, rate, bigEndian);
    }

    /**
     * Set port
     * @param p
     */
    public void setPort(int p) {
        port = p;
    }

    /**
     * Get a mixer object
     * @param info
     * @return
     * @throws Exception
     */
    private TargetDataLine getMixerTarget(DataLine.Info info) throws Exception {
        mixerInfo = AudioSystem.getMixerInfo();
        Mixer mixer = null;

        System.out.println("Available mixers:");
        for (int i = 0; i < mixerInfo.length; i++) {

            mixer = AudioSystem.getMixer(mixerInfo[ i]);
            if (mixer.isLineSupported(info)) {
                System.out.println("Line is supported");
                gMixer = mixer;
                return (TargetDataLine) mixer.getLine(info);
            }
        }// end for loop	
        return null;
    }

    /**
     * 
     * 
     * 
     * Read from the MIC and send to our chat partner...
     * 
     * 
     * 
     * @author MYDEVPLANET.COM
     * 
     */
    class AudioSender extends Thread {

        public AudioSender() {
            // this.dgSocket = s;
        }
        private SourceDataLine sourceDataLine;
        private TargetDataLine targetDataLine;
        private AudioFormat.Encoding encoding = AudioFormat.Encoding.PCM_SIGNED;
        private AudioFormat audioFormat;
        private DataLine.Info info;
        private String strIP = null;
        private int port = 0;
        private int sampleSize = 16;
        private int channels = 1;
        private boolean bigEndian = true;

        /*
         * This runs in its own thread
         */
        public void run() {
            long lBytesOut = 0;
            long lStart = System.currentTimeMillis();

            try {

                DataLine.Info sourceLineInfo = new DataLine.Info(
                        SourceDataLine.class, getFormat(), iBufferSize);

                /*
                 * Source is the OUTPUT device.. here it's the speaker...
                 */

                sourceDataLine = (SourceDataLine) AudioSystem.getLine(sourceLineInfo);



                DataLine.Info targetLineInfo = new DataLine.Info(
                        TargetDataLine.class, getFormat(), iBufferSize);


                targetDataLine = (TargetDataLine) AudioSystem.getLine(targetLineInfo);

                sourceDataLine.open();
                targetDataLine.open();

                sourceDataLine.start();
                targetDataLine.start();

                byte buffer[] = new byte[sourceDataLine.getBufferSize()];

                InetAddress address = InetAddress.getByName(strIP);

                byte[] byteData = new byte[2048];
                int buffersize = byteData.length;

                DatagramPacket packet = new DatagramPacket(byteData, buffersize,
                        address, port);

                dgSocket.setSoTimeout(2500);

                while (!globals.isHangup()) {
                    int read = targetDataLine.read(buffer, 0, buffer.length);
                    packet = new DatagramPacket(buffer, read, address, port);

                    try {
                        dgSocket.send(packet);
                        lBytesOut += packet.getLength();

                        //System.out.println( "Sent..." + packet.getLength() + " " + packet.getAddress().toString() );
                    } catch (Exception e1) {
                        e1.printStackTrace(System.out);
                        continue;
                    }

                    long lElapsed = (System.currentTimeMillis() - lStart) / 1000;
                    if (lElapsed > 0) {
                        globals.getStatusScreen().setStatusOut("Bytes out per second: "
                                + lBytesOut / lElapsed);
                    }
                    Thread.yield();

                }

                dgSocket.close();

            } catch (Exception e) {
                e.printStackTrace(System.out);
            } finally {
                try {
                    dgSocket.close();
                } catch (Exception e) {
                }
            }

        }

        public void setIP(String s) {
            this.strIP = s;
        }

        public void setPort(int p) {
            port = p;
        }

        private AudioFormat getFormat() {
            return new AudioFormat(encoding, rate, sampleSize, channels,
                    (sampleSize / 8) * channels, rate, bigEndian);
        }
    }
}
